==> dasu_1.0.jar: edu/northwestern/dasu/drools/RuleBasedEngine.java <==
/**
 * Dasu Project
 *
 * File:         RuleBasedEngine.java
 * RCS:          $Id: RuleBasedEngine.java,v 1.4 2010/06/04 05:01:25 mas939 Exp $
 * Description:  RuleBasedEngine class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:23:51 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */


package edu.northwestern.dasu.drools;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderConfiguration;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.MainGeneric;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The RuleBasedEngine class ...
 */
public abstract class RuleBasedEngine extends Thread {

	private boolean active = false;
	private boolean initialized = false;
	protected static boolean DEBUG = false;
	protected String filename = null;
	protected int taskId = -1;

	protected KnowledgeBase kbase = null;
	protected StatefulKnowledgeSession ksession = null; 
	protected KnowledgeRuntimeLogger logger = null;

	private Hashtable<Long, FactHandle> facts = new Hashtable<Long, FactHandle>();
	private Set<FactHandle> modifiedFacts = 
		Collections.synchronizedSet(new HashSet<FactHandle>());
	
	@Override
	public synchronized void start() {
		super.start();
	}	
	
	protected RuleBasedEngine(String name) {
		super(name);
		initialize();
	}
	
	protected RuleBasedEngine(String name, String filename) {
		super(name);
		this.filename = filename;
		initialize();
	}
	

    // Add the object and its contents to working memory
	public void addFact(final FactTemplate factObject) {
		
		if (isActive()) {
		
			if (!facts.containsKey(factObject.getId())) {
				FactHandle fh = ksession.insert(factObject);
				facts.put(factObject.getId(), fh);
			
				factObject.addPropertyChangeListener(
						new PropertyChangeListener() {
							public void propertyChange(PropertyChangeEvent e) {
						        updateFact(factObject);
							}
						});
			}
			else 
				updateFact(factObject);
		}
	}
		
	private void updateFact(FactTemplate factObject) {
						
		if (isActive()) {
			FactHandle fh = ksession.getFactHandle(factObject);
			
			if (fh != null) {
				
				/* object is being modified in working memory,
				 * avoid concurrent modification by doing nothing 
				 */

				if (modifiedFacts.contains(fh)) {
					if (DEBUG) {
						System.out.println("!!!!!DEBUG: updateFact(): Fact being modified, skiip" + 
							factObject);
					}
					return;
				}
				
				ksession.update(fh, factObject);
//				ksession.update(facts.get(factObject.getId()), factObject);
			}
			else
				facts.remove(factObject.getId());
		}
	}
	
    // Remove the object and its contents from working memory
	public void retractFact(FactTemplate factObject) {
		if (isActive()) {
			FactHandle fh = ksession.getFactHandle(factObject);
			if (fh != null)
				ksession.retract(fh);
//				ksession.retract(facts.get(factObject.getId()));
			else
				facts.remove(factObject.getId());
		}
			
	}


	/* FireAllRules, GetGlobal, SetGlobal, InsertObject, InsertElements
	Query, StartProcess, BatchExecution */
	
	public void executeCommand(String command) {
	
		System.out.println(command);
		//ksession.execute(command);
		
	}
	
	private void initialize() {
		
		try {

			if (loadRules()) {
				loadFacts();
				setInitialized(true);
			}
			else {
				throw (new Exception("No rules to load... aborting engine startup..."));
			}
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	@Override
	public void run() {
		
		if (isInitialized()) {
			setActive(true);
			if (DEBUG)
				System.out.println(this.getName() + ": Starting execution Drools engine\n\n");
		}

		while (!MainGeneric.isShuttingDown() && isActive()) {
			
			ksession.fireUntilHalt();
			if (DEBUG)
				System.out.println(this.getName() + ": Finishing executiong Drools engine\n\n");

			setActive(false);			
			
		} // end while active
		
		destroySession();

	}
	
	protected void destroySession() {
		ksession.dispose();
	}
	
	protected KnowledgeBase loadRules(String fileName) {

//		String PATH = System.getProperty("java.class.path");
//		System.out.println("PATH PATH: " + System.getProperty("java.class.path"));
//		PATH = PATH.concat(":" + Main.getPluginInterface().getPluginDirectoryName() +
//				java.io.File.separator + 
//				Main.getPluginInterface().getPluginID() +  "_" +
//				Main.getPluginInterface().getPluginVersion() +
//				".jar");		
//		System.setProperty("java.class.path", PATH);
//		System.out.println("PATH PATH: " + System.getProperty("java.class.path"));
//
//		Properties props = new Properties();
//        props.setProperty("drools.dialect.java.compiler", "JANINO");
		
        KnowledgeBuilderConfiguration config =
        KnowledgeBuilderFactory.newKnowledgeBuilderConfiguration(null, 
        		Main.getPluginInterface().getPluginClassLoader());
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder(config);
		
//		kbuilder.add(ResourceFactory.newClassPathResource(fileName), ResourceType.DRL);
		kbuilder.add(ResourceFactory.newFileResource(fileName), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBaseConfiguration conf = 
			KnowledgeBaseFactory.newKnowledgeBaseConfiguration(null, 
					Main.getPluginInterface().getPluginClassLoader());
		kbase = KnowledgeBaseFactory.newKnowledgeBase(conf);
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		
		return kbase;

	}
	
	public boolean loadRules() {
		
		return false;
	
	}
	
	protected void loadFacts() {
		
	}

	public void displayFacts() {
		
		if (isActive()) {
			Iterator<FactHandle> f = ksession.getFactHandles().iterator();
			while (f.hasNext()) {
				FactHandle fh = f.next();
				System.out.println(ksession.getObject(fh).toString());
			}
			System.out.println("\n");
		}
		
	}
	
	
	public void stopEngine() {
		if (isActive()) {
			ksession.halt();
			setActive(false);
		}
	}
	
	public boolean isActive() {
		return active;
	}
	
	public void setActive(boolean val) {
		this.active = val;
	}
	
	public void setInitialized(boolean state) {
		initialized = state;
	}
	
	public boolean isInitialized() {
		return initialized;
	}

	public void setModifiedFacts(Set<FactHandle> modifiedFacts) {
		this.modifiedFacts = modifiedFacts;
	}

	public Set<FactHandle> getModifiedFacts() {
		return modifiedFacts;
	}

	
}

==> dasu_2.02.jar: edu/northwestern/dasu/drools/RuleBasedEngine.java <==
/**
 * Dasu Project
 *
 * File:         RuleBasedEngine.java
 * RCS:          $Id: RuleBasedEngine.java,v 1.7 2010/11/01 15:37:00 mas939 Exp $
 * Description:  RuleBasedEngine class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:23:51 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */


package edu.northwestern.dasu.drools;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderConfiguration;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.MainGeneric;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The RuleBasedEngine class ...
 */
public abstract class RuleBasedEngine extends Thread {

	private boolean active = false;
	private boolean initialized = false;
	protected static boolean DEBUG = false;
	protected String filename = null;
	protected int taskId = -1;

	protected KnowledgeBase kbase = null;
	protected StatefulKnowledgeSession ksession = null; 
	protected KnowledgeRuntimeLogger logger = null;

	private Hashtable<Long, FactHandle> facts = new Hashtable<Long, FactHandle>();
	private Set<FactHandle> modifiedFacts = 
		Collections.synchronizedSet(new HashSet<FactHandle>());
	
	@Override
	public synchronized void start() {
		super.start();
	}	
	
	protected RuleBasedEngine(String name) {
		super(name);
		initialize();
	}
	
	protected RuleBasedEngine(String name, String filename) {
		super(name);
		this.filename = filename;
		initialize();
	}
	

    // Add the object and its contents to working memory
	public void addFact(final FactTemplate factObject) {
		
		if (isActive()) {
		
			if (!facts.containsKey(factObject.getId())) {
				FactHandle fh = ksession.insert(factObject);
				facts.put(factObject.getId(), fh);
			
				factObject.addPropertyChangeListener(
						new PropertyChangeListener() {
							public void propertyChange(PropertyChangeEvent e) {
						        updateFact(factObject);
							}
						});
			}
			else 
				updateFact(factObject);
		}
	}
		
	private void updateFact(FactTemplate factObject) {
						
		if (isActive()) {
			FactHandle fh = ksession.getFactHandle(factObject);
			
			if (fh != null) {
				
				/* object is being modified in working memory,
				 * avoid concurrent modification by doing nothing 
				 */

				if (modifiedFacts.contains(fh)) {
					if (DEBUG) {
						System.out.println("!!!!!DEBUG: updateFact(): Fact being modified, skip: " + 
							factObject);
					}
					return;
				}
				
				ksession.update(fh, factObject);
				//ksession.update(facts.get(factObject.getId()), factObject);
			}
			else
				facts.remove(factObject.getId());
		}
	}
	
    // Remove the object and its contents from working memory
	public void retractFact(FactTemplate factObject) {
		if (isActive()) {
			FactHandle fh = ksession.getFactHandle(factObject);
			if (fh != null)
				ksession.retract(fh);
//				ksession.retract(facts.get(factObject.getId()));
			else
				facts.remove(factObject.getId());
		}
			
	}


	/* FireAllRules, GetGlobal, SetGlobal, InsertObject, InsertElements
	Query, StartProcess, BatchExecution */
	
	public void executeCommand(String command) {
	
		System.out.println(command);
		//ksession.execute(command);
		
	}
	
	private void initialize() {
		
		try {

			if (loadRules()) {
				loadFacts();
				setInitialized(true);
			}
			else {
				throw (new Exception("No rules to load... aborting engine startup..."));
			}
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	@Override
	public void run() {
		
		if (isInitialized()) {
			setActive(true);
			if (DEBUG)
				System.out.println(this.getName() + ": Starting execution Drools engine\n\n");
		}

		while (!MainGeneric.isShuttingDown() && isActive()) {
			
			ksession.fireUntilHalt();
			if (DEBUG)
				System.out.println(this.getName() + ": Finishing execution of Drools engine\n\n");

			setActive(false);			
			
		} // end while active
		
		destroySession();

	}
	
	protected void destroySession() {

		if (ksession != null)
			ksession.dispose();
	}
	
	protected KnowledgeBase loadRules(String fileName) {
//
//		Properties props = new Properties();
//        props.setProperty("drools.dialect.java.compiler", "JANINO");
		
        KnowledgeBuilderConfiguration config =
        KnowledgeBuilderFactory.newKnowledgeBuilderConfiguration(null, 
        		Main.getPluginInterface().getPluginClassLoader());
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder(config);
		
//		kbuilder.add(ResourceFactory.newClassPathResource(fileName), ResourceType.DRL);
		kbuilder.add(ResourceFactory.newFileResource(fileName), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBaseConfiguration conf = 
			KnowledgeBaseFactory.newKnowledgeBaseConfiguration(null, 
					Main.getPluginInterface().getPluginClassLoader());
		kbase = KnowledgeBaseFactory.newKnowledgeBase(conf);
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		
		return kbase;

	}
	
	public boolean loadRules() {
		
		return false;
	
	}
	
	protected void loadFacts() {
		
	}

	public void displayFacts() {
		
		if (isActive()) {
			Iterator<FactHandle> f = ksession.getFactHandles().iterator();
			while (f.hasNext()) {
				FactHandle fh = f.next();
				System.out.println(getName() + " => " + ksession.getObject(fh).toString());
			}
			System.out.println("\n");
		}
		
	}
	
	
	public void stopEngine() {
		if (DEBUG)
			System.out.println("stopEngine()");
		
		if (isActive()) {
			ksession.halt();
			setActive(false);
		}
	}
	
	public boolean isActive() {
		return active;
	}
	
	public void setActive(boolean val) {
		this.active = val;
	}
	
	public void setInitialized(boolean state) {
		initialized = state;
	}
	
	public boolean isInitialized() {
		return initialized;
	}

	public void setModifiedFacts(Set<FactHandle> modifiedFacts) {
		this.modifiedFacts = modifiedFacts;
	}

	public Set<FactHandle> getModifiedFacts() {
		return modifiedFacts;
	}

	
}
